@@ -1,10 +1,10 @@
 (* This code is inspired by mimestring.ml from OcamlNet *)
-(* Copyright Gerd Stolpmann, Patrick Doane *) 
+(* Copyright Gerd Stolpmann, Patrick Doane *)
 (* Modified for Ocsigen/Lwt by Nataliya Guts and Vincent Balat *)
 
 module S = Netstring_pcre
 open Lwt
-open Ocsistream
+open Ocsigen_stream
 
 exception Multipart_error of string
 
@@ -15,7 +15,7 @@ let header_stripped_re =
 
 let header_unstripped_re =
   S.regexp "([^ \t\r\n:]+):([ \t]*.*\n([ \t].*\n)*)";;
-(* This much simpler expression returns the name and the unstripped 
+(* This much simpler expression returns the name and the unstripped
  * value.
  *)
 
@@ -38,9 +38,9 @@ let scan_header ?(downcase=true)
           let i' = S.match_end r in
           if i' > i1 then
             raise (Multipart_error "Mimestring.scan_header");
-          let name = 
+          let name =
             if downcase then
-              String.lowercase(S.matched_group r 1 parstr) 
+              String.lowercase(S.matched_group r 1 parstr)
             else
               S.matched_group r 1 parstr
           in
@@ -70,7 +70,7 @@ let read_header ?downcase ?unfold ?strip s =
   let rec find_end_of_header s =
     catch
       (fun () ->
-        let b = Ocsistream.current_buffer s in
+        let b = Ocsigen_stream.current_buffer s in
         (* Maybe the header is empty. In this case, there is an empty line
          * right at the beginning
          *)
@@ -83,19 +83,19 @@ let read_header ?downcase ?unfold ?strip s =
               (s, (S.match_end (snd (S.search_forward end_of_header_re b 0))))
       )
       (function
-        | Not_found -> 
-            Ocsistream.enlarge_stream s >>= 
+        | Not_found ->
+            Ocsigen_stream.enlarge_stream s >>=
             (function
                 Finished _ -> fail Stream_too_small
               | Cont (stri, _) as s -> find_end_of_header s)
         | e -> fail e)
   in
   find_end_of_header s >>= (fun (s, end_pos) ->
-    let b = Ocsistream.current_buffer s in
-    let header, _ = 
-      scan_header ?downcase ?unfold ?strip b ~start_pos:0 ~end_pos 
+    let b = Ocsigen_stream.current_buffer s in
+    let header, _ =
+      scan_header ?downcase ?unfold ?strip b ~start_pos:0 ~end_pos
     in
-    Ocsistream.skip s end_pos >>= 
+    Ocsigen_stream.skip s end_pos >>=
     (fun s -> return (s, header)))
 ;;
 
@@ -107,10 +107,10 @@ let read_multipart_body decode_part boundary s =
 
   let rec search_window s re start =
     try
-      return (s, snd (S.search_forward re (Ocsistream.current_buffer s) start))
+      return (s, snd (S.search_forward re (Ocsigen_stream.current_buffer s) start))
     with
-      Not_found -> 
-        Ocsistream.enlarge_stream s >>=
+      Not_found ->
+        Ocsigen_stream.enlarge_stream s >>=
         (function
           | Finished _ -> fail Stream_too_small
           | Cont (stri, _) as s -> search_window s re start)
@@ -118,11 +118,11 @@ let read_multipart_body decode_part boundary s =
   let search_end_of_line s k =
     (* Search LF beginning at position k *)
     catch
-      (fun () -> (search_window s lf_re k) >>= 
+      (fun () -> (search_window s lf_re k) >>=
         (fun (s, x) -> return (s, (S.match_end x))))
     (function
       | Not_found ->
-          fail (Multipart_error 
+          fail (Multipart_error
                   "read_multipart_body: MIME boundary without line end")
       | e -> fail e)
   in
@@ -139,7 +139,7 @@ let read_multipart_body decode_part boundary s =
   let check_beginning_is_boundary s =
     let del = "--" ^ boundary in
     let ldel = String.length del in
-    Ocsistream.stream_want s (ldel + 2) >>= (function
+    Ocsigen_stream.stream_want s (ldel + 2) >>= (function
       | Finished _ as str2 -> return (str2, false, false)
       | Cont (ss, f) as str2 ->
           let long = String.length ss in
@@ -149,64 +149,64 @@ let read_multipart_body decode_part boundary s =
   in
 
   let rec parse_parts s uses_crlf =
-    (* PRE: [s] is at the beginning of the next part. 
+    (* PRE: [s] is at the beginning of the next part.
      * [uses_crlf] must be true if CRLF is used as EOL sequence, and false
      *    if only LF is used as EOL sequence.
      *)
     let delimiter = (if uses_crlf then "\r" else "" ) ^ "\n--" ^ boundary in
-    Ocsistream.substream delimiter s >>= fun a ->
+    Ocsigen_stream.substream delimiter s >>= fun a ->
     decode_part a >>= fun (y, s) ->
-    (* Now the position of [s] is at the beginning of the delimiter. 
+    (* Now the position of [s] is at the beginning of the delimiter.
      * Check if there is a "--" after the delimiter (==> last part)
      *)
     let l_delimiter = String.length delimiter in
-    Ocsistream.next s >>= fun s ->
-    Ocsistream.stream_want s (l_delimiter+2) >>= fun s ->
+    Ocsigen_stream.next s >>= fun s ->
+    Ocsigen_stream.stream_want s (l_delimiter+2) >>= fun s ->
     let last_part = match s with
     | Finished _ -> false
     | Cont (ss, f) ->
         let long = String.length ss in
         (long >= (l_delimiter+2)) &&
-        (ss.[l_delimiter] = '-') && 
+        (ss.[l_delimiter] = '-') &&
         (ss.[l_delimiter+1] = '-')
     in
     if last_part then
       return [ y ]
     else begin
-      search_end_of_line s 2 >>= fun (s, k) -> 
-      (* [k]: Beginning of next part *) 
-      Ocsistream.skip s k >>= fun s -> 
-      parse_parts s uses_crlf >>= fun l -> 
+      search_end_of_line s 2 >>= fun (s, k) ->
+      (* [k]: Beginning of next part *)
+      Ocsigen_stream.skip s k >>= fun s ->
+      parse_parts s uses_crlf >>= fun l ->
       return (y :: l)
     end
   in
 
   (* Check whether s directly begins with a boundary: *)
-  check_beginning_is_boundary s >>= fun (s, b, islast) -> 
+  check_beginning_is_boundary s >>= fun (s, b, islast) ->
   if islast then return []
   else
   if b then begin
     (* Move to the beginning of the next line: *)
     search_end_of_line s 0 >>= (fun (s, k_eol) ->
-      let uses_crlf = (Ocsistream.current_buffer s).[k_eol-2] = '\r' in
-      Ocsistream.skip s k_eol >>= fun s ->
+      let uses_crlf = (Ocsigen_stream.current_buffer s).[k_eol-2] = '\r' in
+      Ocsigen_stream.skip s k_eol >>= fun s ->
       (* Begin with first part: *)
       parse_parts s uses_crlf)
   end
   else begin
     (* Search the first boundary: *)
     catch
-      (fun () -> 
+      (fun () ->
         search_first_boundary s >>= fun (s, k_eob) ->   (* or Not_found *)
         (* Printf.printf "k_eob=%d\n" k_eob; *)
         (* Move to the beginning of the next line: *)
         search_end_of_line s k_eob >>= fun (s, k_eol) ->
-          let uses_crlf = (Ocsistream.current_buffer s).[k_eol-2] = '\r' in
+          let uses_crlf = (Ocsigen_stream.current_buffer s).[k_eol-2] = '\r' in
           (* Printf.printf "k_eol=%d\n" k_eol; *)
-          Ocsistream.skip s k_eol >>= fun s ->
+          Ocsigen_stream.skip s k_eol >>= fun s ->
           (* Begin with first part: *)
           parse_parts s uses_crlf)
-      (function 
+      (function
         | Not_found ->
             (* No boundary at all: The body is empty. *)
             return []
@@ -215,7 +215,7 @@ let read_multipart_body decode_part boundary s =
 ;;
 
 let empty_stream =
-  Ocsistream.get (Ocsistream.make (fun () -> Ocsistream.empty None))
+  Ocsigen_stream.get (Ocsigen_stream.make (fun () -> Ocsigen_stream.empty None))
 
 let scan_multipart_body_from_stream s ~boundary ~create ~add ~stop =
   let decode_part stream =
@@ -227,29 +227,29 @@ let scan_multipart_body_from_stream s ~boundary ~create ~add ~stop =
         | Cont (stri, f) ->
             let long = String.length stri in
             let size2 = Int64.add size (Int64.of_int long) in
-            let max = Ocsiconfig.get_maxuploadfilesize () in
+            let max = Ocsigen_config.get_maxuploadfilesize () in
             if
               (match max with
                 None -> false
               | Some m ->
                   (Int64.compare size2 m) > 0)
-            then 
-              fail (Ocsimisc.Ocsigen_Request_too_long)
+            then
+              fail (Ocsigen_lib.Ocsigen_Request_too_long)
             else
               if stri = ""
-              then Ocsistream.next f >>= while_stream size
-              else ((* catch 
-                       (fun () -> 
+              then Ocsigen_stream.next f >>= while_stream size
+              else ((* catch
+                       (fun () ->
                          add p stri)
-                       (fun e -> f () >>= 
-                         Ocsistream.consume >>= 
+                       (fun e -> f () >>=
+                         Ocsigen_stream.consume >>=
                          (fun () -> fail e)) *)
                   add p stri >>= fun () ->
-                Ocsistream.next f >>= 
+                Ocsigen_stream.next f >>=
                 while_stream size2)
-      in 
-      catch 
-        (fun () -> while_stream Int64.zero s >>= 
+      in
+      catch
+        (fun () -> while_stream Int64.zero s >>=
           (fun (size, s) -> stop size p >>= fun r -> return (r, s)))
         (function
             error -> stop Int64.zero p >>= fun _ -> fail error))
@@ -257,11 +257,11 @@ let scan_multipart_body_from_stream s ~boundary ~create ~add ~stop =
   catch
     (fun () ->
       (* read the multipart body: *)
-      Ocsistream.next s >>= fun s ->
+      Ocsigen_stream.next s >>= fun s ->
       read_multipart_body decode_part boundary s >>=
       (fun _ -> return ()))
     (function
-      | Stream_too_small -> fail Ocsimisc.Ocsigen_Bad_Request
+      | Stream_too_small -> fail Ocsigen_lib.Ocsigen_Bad_Request
       | e -> fail e)
 ;;
 
