@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, with linking exception; 
+ * the Free Software Foundation, with linking exception;
  * either version 2.1 of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -23,5 +23,7 @@ type xml =
   | PCData of string ];
 (**/**)
 exception Xml_parser_error of string;
-value xmlparser : string -> list xml;
+value xmlparser_string : string -> list xml;
+value xmlparser_file : string -> list xml;
+
 
